@@ -37,6 +37,7 @@
 
 #include <cstdio>
 #include <cerrno>
+#include <sys/param.h>
 
 #include <nuttx/nx/nxbe.h>
 
@@ -64,10 +65,6 @@
 
 #define makemult(a,b) ((b == 1) ? (a) : (((int)((a) / (b))) * (b)))
 
-#ifndef MIN
-#  define MIN(a,b)    ((a) < (b) ? (a) : (b))
-#endif
-
 /////////////////////////////////////////////////////////////////////////////
 // Class Implementations
 /////////////////////////////////////////////////////////////////////////////
