@@ -13,8 +13,8 @@
  * The above copyright notice and this permission notice shall be included in
  * all copies or substantial portions of the Software.
  *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
  * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
@@ -136,7 +136,9 @@ static void format_double(struct String *buf, double value, int width,
       else                      /* print decimal numbers or integers, if
                                  * possible */
         {
-          int o, n, p = 6;
+          int o;
+          int n;
+          int p = 6;
 
           while (x >= 10.0 && p > 0)
             {
@@ -153,6 +155,7 @@ static void format_double(struct String *buf, double value, int width,
                 {
                   --buf->length;
                 }
+
               if (buf->character[buf->length - 1] == '.')
                 {
                   --buf->length;
@@ -222,8 +225,8 @@ struct Value *Value_new_NIL(struct Value *this)
   return this;
 }
 
-struct Value *Value_new_ERROR(struct Value *this, int code, const char *error,
-                              ...)
+struct Value *Value_new_ERROR(struct Value *this, int code,
+                              const char *error, ...)
 {
   va_list ap;
   char buf[128];
@@ -234,7 +237,7 @@ struct Value *Value_new_ERROR(struct Value *this, int code, const char *error,
   va_end(ap);
   this->type = V_ERROR;
   this->u.error.code = code;
-  this->u.error.msg = strcpy(malloc(strlen(buf) + 1), buf);
+  this->u.error.msg  = strdup(buf);
   return this;
 }
 
@@ -364,13 +367,12 @@ struct Value *Value_clone(struct Value *this, const struct Value *original)
 {
   assert(this != (struct Value *)0);
   assert(original != (struct Value *)0);
+
   switch (original->type)
     {
     case V_ERROR:
       {
-        strcpy(this->u.error.msg =
-               malloc(strlen(original->u.error.msg) + 1),
-               original->u.error.msg);
+        this->u.error.msg = strdup(original->u.error.msg);
         this->u.error.code = original->u.error.code;
         break;
       }
@@ -1152,6 +1154,7 @@ struct Value *Value_eq(struct Value *this, struct Value *x, int calc)
         Value_new_INTEGER(this, v);
         break;
       }
+
     case V_STRING:
       {
         int v;
@@ -1357,7 +1360,8 @@ struct Value *Value_ne(struct Value *this, struct Value *x, int calc)
   return this;
 }
 
-int Value_exitFor(struct Value *this, struct Value *limit, struct Value *step)
+int Value_exitFor(struct Value *this,
+                  struct Value *limit, struct Value *step)
 {
   switch (this->type)
     {
@@ -1384,7 +1388,8 @@ int Value_exitFor(struct Value *this, struct Value *limit, struct Value *step)
 
 void Value_errorPrefix(struct Value *this, const char *prefix)
 {
-  size_t prefixlen, msglen;
+  size_t prefixlen;
+  size_t msglen;
 
   assert(this->type == V_ERROR);
   prefixlen = strlen(prefix);
@@ -1396,7 +1401,8 @@ void Value_errorPrefix(struct Value *this, const char *prefix)
 
 void Value_errorSuffix(struct Value *this, const char *suffix)
 {
-  size_t suffixlen, msglen;
+  size_t suffixlen;
+  size_t msglen;
 
   assert(this->type == V_ERROR);
   suffixlen = strlen(suffix);
@@ -1410,7 +1416,8 @@ struct Value *Value_new_typeError(struct Value *this, enum ValueType t1,
 {
   assert(typestr[t1]);
   assert(typestr[t2]);
-  return Value_new_ERROR(this, TYPEMISMATCH1, _(typestr[t1]), _(typestr[t2]));
+  return Value_new_ERROR(this, TYPEMISMATCH1,
+                         _(typestr[t1]), _(typestr[t2]));
 }
 
 static void retypeError(struct Value *this, enum ValueType to)
@@ -1617,8 +1624,8 @@ struct String *Value_toString(struct Value *this, struct String *s, char pad,
           {
             if (precision > 0 || exponent)
               {
-                format_double(&buf, (double)this->u.integer, width, precision,
-                              exponent);
+                format_double(&buf, (double)this->u.integer, width,
+                              precision, exponent);
               }
             else if (precision == 0)
               {
@@ -1796,12 +1803,14 @@ struct Value *Value_toStringUsing(struct Value *this, struct String *s,
 
             break;
           }
+
         case '&':              /* output string */
           {
             width = 0;
             ++(*usingpos);
             goto work;
           }
+
         case '*':
         case '$':
         case '0':
@@ -1850,10 +1859,12 @@ struct Value *Value_toStringUsing(struct Value *this, struct String *s,
                   default:
                     ++width;
                   }
+
                 ++(*usingpos);
               }
 
-            if (*usingpos < using->length && using->character[*usingpos] == '.')
+            if (*usingpos < using->length &&
+                using->character[*usingpos] == '.')
               {
                 ++(*usingpos);
                 ++width;
@@ -1873,13 +1884,15 @@ struct Value *Value_toStringUsing(struct Value *this, struct String *s,
                   }
               }
 
-            if (*usingpos < using->length && using->character[*usingpos] == '-')
+            if (*usingpos < using->length &&
+                using->character[*usingpos] == '-')
               {
                 ++(*usingpos);
                 if (headingsign == 0)
                   {
                     headingsign = 2;
                   }
+
                 trailingsign = -1;
               }
             else if (*usingpos < using->length &&
@@ -1890,6 +1903,7 @@ struct Value *Value_toStringUsing(struct Value *this, struct String *s,
                   {
                     headingsign = 2;
                   }
+
                 trailingsign = 1;
               }
 
@@ -1911,8 +1925,8 @@ struct Value *Value_toStringUsing(struct Value *this, struct String *s,
     }
 
 work:
-  Value_toString(this, s, pad, headingsign, width, commas, dollar, dollarleft,
-                 precision, exponent, trailingsign);
+  Value_toString(this, s, pad, headingsign, width, commas, dollar,
+                 dollarleft, precision, exponent, trailingsign);
   if ((this->type == V_INTEGER || this->type == V_REAL) && width == 0 &&
       precision == -1)
     {
@@ -1970,7 +1984,8 @@ struct String *Value_toWrite(struct Value *this, struct String *s)
       {
         double x;
         int p = DBL_DIG;
-        int n, o;
+        int n;
+        int o;
 
         x = (this->u.real < 0.0 ? -this->u.real : this->u.real);
         while (x > 1.0 && p > 0)
@@ -2027,10 +2042,21 @@ struct String *Value_toWrite(struct Value *this, struct String *s)
 
 struct Value *Value_nullValue(enum ValueType type)
 {
-  static struct Value integer = { V_INTEGER };
-  static struct Value real = { V_REAL };
-  static struct Value string = { V_STRING };
-  static char n[] = "";
+  static struct Value integer =
+  {
+    V_INTEGER
+  };
+
+  static struct Value real =
+  {
+    V_REAL
+  };
+
+  static struct Value string =
+  {
+    V_STRING
+  };
+
   static int init = 0;
 
   if (!init)
@@ -2038,7 +2064,8 @@ struct Value *Value_nullValue(enum ValueType type)
       integer.u.integer = 0;
       real.u.real = 0.0;
       string.u.string.length = 0;
-      string.u.string.character = n;
+      string.u.string.character = "";
+      init = 1;
     }
 
   switch (type)
