@@ -1096,7 +1096,7 @@ static int dofor(void)
     }
   else
     {
-      strcpy(g_forstack[nfors].id, id);
+      strlcpy(g_forstack[nfors].id, id, sizeof(g_forstack[nfors].id));
       g_forstack[nfors].nextline = getnextline(g_string);
       g_forstack[nfors].step = stepval;
       g_forstack[nfors].toval = toval;
@@ -2507,7 +2507,8 @@ static FAR struct mb_variable_s *addfloat(FAR const char *id)
   if (vars)
     {
       g_variables = vars;
-      strcpy(g_variables[g_nvariables].id, id);
+      strlcpy(g_variables[g_nvariables].id, id,
+              sizeof(g_variables[g_nvariables].id));
       g_variables[g_nvariables].dval = 0.0;
       g_variables[g_nvariables].sval = NULL;
       g_nvariables++;
@@ -2540,7 +2541,8 @@ static FAR struct mb_variable_s *addstring(FAR const char *id)
   if (vars)
     {
       g_variables = vars;
-      strcpy(g_variables[g_nvariables].id, id);
+      strlcpy(g_variables[g_nvariables].id, id,
+              sizeof(g_variables[g_nvariables].id));
       g_variables[g_nvariables].sval = NULL;
       g_variables[g_nvariables].dval = 0.0;
       g_nvariables++;
@@ -2573,7 +2575,8 @@ static FAR struct mb_dimvar_s *adddimvar(FAR const char *id)
   if (vars)
     {
       g_dimvariables = vars;
-      strcpy(g_dimvariables[g_ndimvariables].id, id);
+      strlcpy(g_dimvariables[g_ndimvariables].id, id,
+              sizeof(g_dimvariables[g_ndimvariables].id));
       g_dimvariables[g_ndimvariables].dval  = NULL;
       g_dimvariables[g_ndimvariables].str   = NULL;
       g_dimvariables[g_ndimvariables].ndims = 0;
@@ -2970,7 +2973,7 @@ static FAR char *stringstring(void)
 
   for (i = 0; i < N; i++)
     {
-      strcpy(answer + len * i, str);
+      strlcpy(answer + len * i, str, (N - i) * len + 1);
     }
 
   free(str);
@@ -4009,15 +4012,7 @@ static int mystrcount(FAR const char *str, char ch)
 
 static FAR char *mystrdup(FAR const char *str)
 {
-  FAR char *answer;
-
-  answer = malloc(strlen(str) + 1);
-  if (answer)
-    {
-      strcpy(answer, str);
-    }
-
-  return answer;
+  return strdup(str);
 }
 
 /****************************************************************************
@@ -4036,11 +4031,11 @@ static FAR char *mystrconcat(FAR const char *str, FAR const char *cat)
   int len;
   FAR char *answer;
 
-  len = strlen(str) + strlen(cat);
-  answer = malloc(len + 1);
+  len = strlen(str) + strlen(cat) + 1;
+  answer = malloc(len);
   if (answer)
     {
-      strcpy(answer, str);
+      strlcpy(answer, str, len);
       strcat(answer, cat);
     }
 
