@@ -23,9 +23,7 @@
 
 #define TOYWASM_USE_SEPARATE_EXECUTE
 #define TOYWASM_USE_TAILCALL
-/* #undef TOYWASM_ENABLE_TRACING */
 #define TOYWASM_USE_JUMP_BINARY_SEARCH
-/* #undef TOYWASM_USE_JUMP_CACHE */
 #define TOYWASM_JUMP_CACHE2_SIZE 4
 #define TOYWASM_USE_LOCALS_CACHE
 #define TOYWASM_USE_SEPARATE_LOCALS
@@ -33,5 +31,11 @@
 #define TOYWASM_USE_RESULTTYPE_CELLIDX
 #define TOYWASM_USE_LOCALTYPE_CELLIDX
 #define TOYWASM_ENABLE_WRITER
+#define TOYWASM_ENABLE_WASM_EXTENDED_CONST
+#define TOYWASM_ENABLE_WASM_MULTI_MEMORY
+#define TOYWASM_ENABLE_WASM_TAILCALL
+#define TOYWASM_ENABLE_WASM_THREADS
+#define TOYWASM_ENABLE_WASI
+#define TOYWASM_ENABLE_WASI_THREADS
 
 #endif /* !defined(_TOYWASM_CONFIG_H) */
