@@ -231,7 +231,7 @@ static FAR uint8_t *dhcpc_addend(FAR uint8_t *optptr)
 static int dhcpc_sendmsg(FAR struct dhcpc_state_s *pdhcpc,
                          FAR struct dhcpc_state *presult, int msgtype)
 {
-  char hostname[HOST_NAME_MAX];
+  char hostname[HOST_NAME_MAX + 1];
   struct sockaddr_in addr;
   FAR uint8_t *pend;
   in_addr_t serverid = INADDR_BROADCAST;
@@ -257,7 +257,7 @@ static int dhcpc_sendmsg(FAR struct dhcpc_state_s *pdhcpc,
 
   if (gethostname(hostname, sizeof(hostname)) || (0 == strlen(hostname)))
     {
-      strncpy(hostname, CONFIG_NETUTILS_DHCPC_HOST_NAME, HOST_NAME_MAX);
+      strlcpy(hostname, CONFIG_NETUTILS_DHCPC_HOST_NAME, sizeof(hostname));
     }
 
   /* Handle the message specific settings */
@@ -860,7 +860,8 @@ int dhcpc_request(FAR void *handle, FAR struct dhcpc_state *presult)
                * that we requested from.
                */
 
-              else if (msgtype == DHCPOFFER)
+              else if (msgtype == DHCPOFFER &&
+                       pdhcpc->serverid.s_addr != presult->serverid.s_addr)
                 {
                   ninfo("Received another OFFER, send DECLINE\n");
                   dhcpc_sendmsg(pdhcpc, presult, DHCPDECLINE);
