@@ -47,6 +47,7 @@
 #include <sys/ioctl.h>
 
 #include <inttypes.h>
+#include <sched.h>
 #include <stdint.h>
 #include <stdbool.h>
 #include <string.h>
@@ -308,7 +309,8 @@ static struct dhcpd_daemon_s g_dhcpd_daemon =
  * Name: dhcpd_arpupdate
  ****************************************************************************/
 
-#ifndef CONFIG_NETUTILS_DHCPD_HOST
+#ifndef CONFIG_NETUTILS_DHCPD_IGNOREBROADCAST
+#  ifndef CONFIG_NETUTILS_DHCPD_HOST
 static inline void dhcpd_arpupdate(FAR uint8_t *ipaddr, FAR uint8_t *hwaddr)
 {
   struct sockaddr_in inaddr;
@@ -323,10 +325,11 @@ static inline void dhcpd_arpupdate(FAR uint8_t *ipaddr, FAR uint8_t *hwaddr)
 
   /* Update the ARP table */
 
-  netlib_set_arpmapping(&inaddr, hwaddr);
+  netlib_set_arpmapping(&inaddr, hwaddr, NULL);
 }
-#else
-#  define dhcpd_arpupdate(ipaddr,hwaddr)
+#  else
+#    define dhcpd_arpupdate(ipaddr,hwaddr)
+#  endif
 #endif
 
 /****************************************************************************
