@@ -27,6 +27,7 @@
 #include <nuttx/mtd/configdata.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
+#include <unistd.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -565,9 +566,9 @@ static void cfgdatacmd_show_all_config_items(void)
   printf(fmtstr, "Name", "Len");
   sprintf(fmtstr, "%%-%ds%%-6d", CONFIG_MTD_CONFIG_NAME_LEN);
 #else
-  strcpy(fmtstr, "%-6s%-6s%-6sData\n");
+  strlcpy(fmtstr, "%-6s%-6s%-6sData\n", sizeof(fmtstr));
   printf(fmtstr, "ID", "Inst", "Len");
-  strcpy(fmtstr, "%-6d%-6d%-6d");
+  strlcpy(fmtstr, "%-6d%-6d%-6d", sizeof(fmtstr));
 #endif
 
   /* Get the first config item */
@@ -619,7 +620,7 @@ static void cfgdatacmd_show_all_config_items(void)
 #ifdef CONFIG_MTD_CONFIG_NAMED
           sprintf(fmtstr2, "\n%ds", CONFIG_MTD_CONFIG_NAME_LEN + 6);
 #else
-          strcpy(fmtstr2, "\n%18s");
+          strlcpy(fmtstr2, "\n%18s", sizeof(fmtstr2));
 #endif
           /* Loop though all bytes and display them */
 
