@@ -35,12 +35,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 #include <nuttx/queue.h>
 #include <nuttx/video/video.h>
 #include <nuttx/video/fb.h>
 #include <sys/ioctl.h>
 #include <sys/mman.h>
+#include <sys/param.h>
 #include <sys/types.h>
 
 #include <system/nxcamera.h>
@@ -58,10 +60,6 @@
 #define NXCAMERA_STATE_LOOPING   2
 #define NXCAMERA_STATE_PAUSED    3
 
-#ifndef MIN
-#  define MIN(a, b)              (((a) < (b)) ? (a) : (b))
-#endif
-
 #define convert_frame            ConvertToARGB
 
 /****************************************************************************
@@ -167,6 +165,36 @@ static int nxcamera_opendevice(FAR struct nxcamera_s *pcam)
   return -ENODEV;
 }
 
+/****************************************************************************
+ * Name: nxcameraer_jointhread
+ ****************************************************************************/
+
+static void nxcamera_jointhread(FAR struct nxcamera_s *pcam)
+{
+  FAR void *value;
+  int id = 0;
+
+  if (gettid() == pcam->loop_id)
+    {
+      return;
+    }
+
+  pthread_mutex_lock(&pcam->mutex);
+
+  if (pcam->loop_id > 0)
+    {
+      id = pcam->loop_id;
+      pcam->loop_id = 0;
+    }
+
+  pthread_mutex_unlock(&pcam->mutex);
+
+  if (id > 0)
+    {
+      pthread_join(id, &value);
+    }
+}
+
 /****************************************************************************
  * Name: nxcamera_loopthread
  *
@@ -477,7 +505,6 @@ int nxcamera_setfile(FAR struct nxcamera_s *pcam, FAR const char *pfile,
 int nxcamera_stop(FAR struct nxcamera_s *pcam)
 {
   struct video_msg_s term_msg;
-  FAR void           *value;
 
   DEBUGASSERT(pcam != NULL);
 
@@ -501,8 +528,7 @@ int nxcamera_stop(FAR struct nxcamera_s *pcam)
 
   /* Join the thread.  The thread will do all the cleanup. */
 
-  pthread_join(pcam->loop_id, &value);
-  pcam->loop_id = 0;
+  nxcamera_jointhread(pcam);
 
   return OK;
 }
@@ -536,7 +562,6 @@ int nxcamera_stream(FAR struct nxcamera_s *pcam,
   struct mq_attr             attr;
   struct sched_param         sparam;
   pthread_attr_t             tattr;
-  FAR void                   *value;
   int                        ret;
   int                        i;
   struct v4l2_buffer         buf;
@@ -681,10 +706,7 @@ int nxcamera_stream(FAR struct nxcamera_s *pcam,
    * to perform clean-up.
    */
 
-  if (pcam->loop_id != 0)
-    {
-      pthread_join(pcam->loop_id, &value);
-    }
+  nxcamera_jointhread(pcam);
 
   pthread_attr_init(&tattr);
   sparam.sched_priority = sched_get_priority_max(SCHED_FIFO) - 9;
@@ -792,24 +814,16 @@ FAR struct nxcamera_s *nxcamera_create(void)
 
 void nxcamera_release(FAR struct nxcamera_s *pcam)
 {
-  FAR void *value;
   int      refcount;
 
+  /* Check if there was a previous thread and join it if there was */
+
+  nxcamera_jointhread(pcam);
+
   /* Lock the mutex */
 
   pthread_mutex_lock(&pcam->mutex);
 
-  /* Check if there was a previous thread and join it if there was */
-
-  if (pcam->loop_id != 0)
-    {
-      pthread_mutex_unlock(&pcam->mutex);
-      pthread_join(pcam->loop_id, &value);
-      pcam->loop_id = 0;
-
-      pthread_mutex_lock(&pcam->mutex);
-    }
-
   /* Reduce the reference count */
 
   refcount = pcam->crefs--;
