@@ -32,6 +32,7 @@
 #include <syslog.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <unistd.h>
 
 #include <nuttx/sched_note.h>
 
@@ -39,17 +40,10 @@
  * Pre-processor Definitions
  ************************************************************************************/
 
-#ifdef CONFIG_SCHED_INSTRUMENTATION_HIRES
-#  define syslog_time(priority, fmt, ...) \
-            syslog(priority, "%08lx.%08lx: " fmt, \
-                   (unsigned long)systime_sec, (unsigned long)systime_nsec, \
-                   __VA_ARGS__)
-#else
 #  define syslog_time(priority, fmt, ...) \
             syslog(priority, "%08lx: " fmt, \
                    (unsigned long)systime, \
                    __VA_ARGS__)
-#endif
 
 /************************************************************************************
  * Private Data
@@ -108,12 +102,8 @@ static void trace_dump_unflatten(FAR void *dst,
 static void dump_notes(size_t nread)
 {
   FAR struct note_common_s *note;
-#ifdef CONFIG_SCHED_INSTRUMENTATION_HIRES
   uint32_t systime_sec;
   uint32_t systime_nsec;
-#else
-  uint32_t systime;
-#endif
   pid_t pid;
   off_t offset;
 
@@ -122,14 +112,10 @@ static void dump_notes(size_t nread)
     {
       note    = (FAR struct note_common_s *)&g_note_buffer[offset];
       trace_dump_unflatten(&pid, note->nc_pid, sizeof(pid));
-#ifdef CONFIG_SCHED_INSTRUMENTATION_HIRES
       trace_dump_unflatten(&systime_nsec,
                            note->nc_systime_nsec, sizeof(systime_nsec));
       trace_dump_unflatten(&systime_sec,
                            note->nc_systime_sec, sizeof(systime_sec));
-#else
-      trace_dump_unflatten(&systime, note->nc_systime, sizeof(systime));
-#endif
 
       switch (note->nc_type)
         {
@@ -787,12 +773,12 @@ static int note_daemon(int argc, char *argv[])
 
   /* Open the note driver */
 
-  syslog(LOG_INFO, "note_daemon: Opening /dev/note\n");
-  fd = open("/dev/note", O_RDONLY);
+  syslog(LOG_INFO, "note_daemon: Opening /dev/note/ram\n");
+  fd = open("/dev/note/ram", O_RDONLY);
   if (fd < 0)
     {
       int errcode = errno;
-      syslog(LOG_ERR, "note_daemon: ERROR: Failed to open /dev/note: %d\n",
+      syslog(LOG_ERR, "note_daemon: ERROR: Failed to open /dev/note/ram: %d\n",
              errcode);
       goto errout;
     }
