@@ -24,13 +24,15 @@
 
 #include <nuttx/config.h>
 
-#include <stdio.h>
-#include <time.h>
+#include <assert.h>
+#include <errno.h>
 #include <fcntl.h>
+#include <mqueue.h>
 #include <pthread.h>
 #include <signal.h>
-#include <mqueue.h>
-#include <errno.h>
+#include <stdio.h>
+#include <time.h>
+#include <unistd.h>
 
 #include "ostest.h"
 
